import React, { useState, useEffect, useRef } from 'react';
import { 
  TextField, 
  Button, 
  Container, 
  Avatar, 
  Tooltip, 
  Alert, 
  CircularProgress,
  Box,
  Typography
} from '@mui/material';
import { PRIMARY_COLOR, SECONDARY_COLOR } from '../App';
import { useApiCall } from './hooks/useApiCall';
import { api } from '../services/api';
import TagManager from './tagManager';

export default function ItemDetails({ item: initialItem, token, onSave, onClose }) {
  const [item, setItem] = useState(initialItem);
  const [name, setName] = useState(initialItem.name);
  const [description, setDescription] = useState(initialItem.description);
  const [imagePath, setImagePath] = useState(initialItem.image_path || '');
  const [imageSrc, setImageSrc] = useState('/images/default.jpg');
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState(null);
  const fileInputRef = useRef(null);
  const [imageOverlayVisible, setImageOverlayVisible] = useState(false);
  const [boxes, setBoxes] = useState([]);
  const [selectedBoxId, setSelectedBoxId] = useState(initialItem.box_id);

  const { execute: fetchBoxes, loading: loadingBoxes, error: boxesError } = useApiCall();
  const { execute: fetchItem, loading: loadingItem } = useApiCall();
  const { execute: updateItem, loading: savingItem, error: saveError } = useApiCall();
  const { execute: uploadImage, loading: uploadingImage, error: uploadError } = useApiCall();

  // Single effect to load initial data
  useEffect(() => {
    const loadInitialData = async () => {
      try {
        // Fetch item and boxes in parallel
        const [itemResponse, boxesResponse] = await Promise.all([
          fetchItem(() => api.items.getOne(token, initialItem.ID)),
          fetchBoxes(() => api.boxes.getAll(token))
        ]);

        const updatedItem = itemResponse.data;
        setItem(updatedItem);
        setName(updatedItem.name);
        setDescription(updatedItem.description);
        setImagePath(updatedItem.image_path || '');
        setBoxes(boxesResponse.data);

        // Load image
        try {
          const imageResponse = await api.items.getImage(token, initialItem.ID);
          const reader = new FileReader();
          reader.onload = () => {
            setImageSrc(reader.result);
            setLoading(false);
          };
          reader.readAsDataURL(imageResponse.data);
        } catch (err) {
          setImageSrc('/default.jpg');
          setLoading(false);
        }
      } catch (err) {
        setError(err);
        setLoading(false);
      }
    };

    loadInitialData();
  }, []); // Empty dependency array - only run once on mount

  const handleImageUpload = async () => {
    if (!fileInputRef.current?.files?.[0]) return null;

    const formData = new FormData();
    formData.append('image', fileInputRef.current.files[0]);

    try {
      const response = await uploadImage(() => 
        api.items.uploadImage(token, item.ID, formData)
      );
      return response.data.imagePath;
    } catch (err) {
      return null;
    }
  };

  const handleSave = async () => {
    if (fileInputRef.current?.files?.[0]) {
      await handleImageUpload();
    }

    try {
      await updateItem(() => api.items.update(token, item.ID, {
        name,
        description,
        box_id: +selectedBoxId,
      }));
      onSave();
    } catch (err) {}
  };

  const handleBoxChange = (event) => {
    setSelectedBoxId(event.target.value);
  };

  const handleTagsChange = (newTags) => {
    setItem(prev => ({...prev, tags: newTags}));
  };

  return (
    <Container>
      <h3>Edit Item: {name}</h3>
      
      {(error || boxesError || saveError || uploadError) && (
        <Alert severity="error" sx={{ mb: 2 }}>
          {error?.message || boxesError?.message || saveError?.message || uploadError?.message}
        </Alert>
      )}

      {loading ? (
        <CircularProgress />
      ) : (
        /* Rest of your JSX remains the same */
        // ... existing JSX code ...
        <Box>
          {/* Image section */}
          <Tooltip title="Click to enlarge">
            <Avatar
              src={imageSrc}
              alt={name}
              sx={{ width: 200, height: 200, marginBottom: '16px' }}
              onClick={() => setImageOverlayVisible(true)}
            />
          </Tooltip>

          {/* Form fields */}
          <TextField
            label="Item Name"
            variant="outlined"
            fullWidth
            margin="normal"
            value={name}
            onChange={(e) => setName(e.target.value)}
            disabled={savingItem}
          />
          
          <TextField
            label="Item Description"
            variant="outlined"
            fullWidth
            margin="normal"
            value={description}
            onChange={(e) => setDescription(e.target.value)}
            disabled={savingItem}
          />

          {/* Box selection */}
          {loadingBoxes ? (
            <CircularProgress size={24} />
          ) : (
            <select 
              value={selectedBoxId} 
              onChange={handleBoxChange}
              disabled={savingItem}
            >
              <option value="">No box</option>
              {boxes.map((box) => (
                <option key={box.ID} value={box.ID}>
                  {box.name}
                </option>
              ))}
            </select>
          )}

          {/* Tags section */}
          <Box sx={{ my: 2 }}>
            <Typography variant="subtitle1" gutterBottom>Tags</Typography>
            <TagManager 
              token={token} 
              itemId={item.ID}
              initialTags={item.tags || []}
              onTagsChange={handleTagsChange}
            />
          </Box>

          {/* Buttons */}
          <Box sx={{ mt: 2 }}>
            <Button 
              sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR, mr: 1 }}
              variant="contained" 
              component="label"
              disabled={uploadingImage || savingItem}
            >
              {uploadingImage ? <CircularProgress size={24} /> : 'Upload Image'}
              <input
                type="file"
                accept="image/*"
                ref={fileInputRef}
                style={{ display: 'none' }}
                onChange={(e) => setImagePath(e.target.files[0]?.name || '')}
              />
            </Button>

            <Button 
              sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR, mr: 1 }}
              variant="contained"
              onClick={handleSave}
              disabled={savingItem}
            >
              {savingItem ? <CircularProgress size={24} /> : 'Save Changes'}
            </Button>

            <Button 
              sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }}
              variant="contained"
              onClick={onClose}
              disabled={savingItem}
            >
              Close
            </Button>
          </Box>
        </Box>
      )}
    </Container>
  );
}