import React, { useState, useEffect, useCallback } from 'react';
import { 
  Chip,
  Dialog,
  DialogTitle,
  DialogContent,
  DialogActions,
  TextField,
  Button,
  Box,
  CircularProgress,
  Alert
} from '@mui/material';
import { LocalOffer as TagIcon } from '@mui/icons-material';
import { useApiCall } from './hooks/useApiCall';
import { api } from '../services/api';

const TagManager = React.memo(({ token, itemId, initialTags = [], onTagsChange }) => {
  const [tags, setTags] = useState([]);
  const [itemTags, setItemTags] = useState(initialTags);
  const [openDialog, setOpenDialog] = useState(false);
  const [newTagName, setNewTagName] = useState('');
  const [newTagDescription, setNewTagDescription] = useState('');
  const [newTagColor, setNewTagColor] = useState('#3366ff');

  const { execute: fetchTags, loading: loadingTags, error: tagsError } = useApiCall();
  const { execute: createTag, loading: creatingTag, error: createError } = useApiCall();
  const { execute: addTags, loading: addingTags, error: addError } = useApiCall();
  const { execute: removeTag, loading: removingTag, error: removeError } = useApiCall();

  // Only fetch tags once when component mounts
  useEffect(() => {
    const getTags = async () => {
      try {
        const response = await fetchTags(() => api.tags.getAll(token));
        setTags(response.data);
      } catch (err) {}
    };
    getTags();
  }, []); // Empty dependency array - only run once on mount

  // Update itemTags when initialTags prop changes
  useEffect(() => {
    setItemTags(initialTags);
  }, [initialTags]);

  const handleCreateTag = async () => {
    try {
      const response = await createTag(() => api.tags.create(token, {
        name: newTagName,
        description: newTagDescription,
        color: newTagColor
      }));
      setTags(prevTags => [...prevTags, response.data]);
      setOpenDialog(false);
      setNewTagName('');
      setNewTagDescription('');
      setNewTagColor('#3366ff');
    } catch (err) {}
  };

  const handleAddTag = async (tagId) => {
    if (!itemId) return;
    try {
      await addTags(() => api.items.addTags(token, itemId, [tagId]));
      const newTag = tags.find(t => t.ID === tagId);
      if (newTag) {
        const updatedTags = [...itemTags, newTag];
        setItemTags(updatedTags);
        onTagsChange?.(updatedTags);
      }
    } catch (err) {}
  };

  const handleRemoveTag = async (tagId) => {
    if (!itemId) return;
    try {
      await removeTag(() => api.items.removeTag(token, itemId, tagId));
      const updatedTags = itemTags.filter(tag => tag.ID !== tagId);
      setItemTags(updatedTags);
      onTagsChange?.(updatedTags);
    } catch (err) {}
  };

  return (
    <Box>
      {(tagsError || createError || addError || removeError) && (
        <Alert severity="error" sx={{ mb: 2 }}>
          {tagsError?.message || createError?.message || 
           addError?.message || removeError?.message}
        </Alert>
      )}

      <Box display="flex" alignItems="center" flexWrap="wrap" gap={1}>
        {itemTags.map(tag => (
          <Chip
            key={tag.ID}
            label={tag.name}
            onDelete={() => handleRemoveTag(tag.ID)}
            style={{ 
              backgroundColor: tag.color,
              color: '#fff'
            }}
            disabled={removingTag}
          />
        ))}

        <Box sx={{ ml: 1 }}>
          {loadingTags ? (
            <CircularProgress size={24} />
          ) : (
            tags
              .filter(tag => !itemTags.find(it => it.ID === tag.ID))
              .map(tag => (
                <Chip
                  key={tag.ID}
                  label={tag.name}
                  onClick={() => handleAddTag(tag.ID)}
                  style={{ 
                    backgroundColor: tag.color,
                    color: '#fff',
                    margin: '0 4px'
                  }}
                  disabled={addingTags}
                />
              ))
          )}
        </Box>

        <Button
          startIcon={<TagIcon />}
          variant="outlined"
          size="small"
          onClick={() => setOpenDialog(true)}
          sx={{ ml: 1 }}
        >
          New Tag
        </Button>
      </Box>

      <Dialog open={openDialog} onClose={() => setOpenDialog(false)}>
        <DialogTitle>Create New Tag</DialogTitle>
        <DialogContent>
          <TextField
            label="Name"
            value={newTagName}
            onChange={(e) => setNewTagName(e.target.value)}
            fullWidth
            margin="normal"
          />
          <TextField
            label="Description"
            value={newTagDescription}
            onChange={(e) => setNewTagDescription(e.target.value)}
            fullWidth
            margin="normal"
          />
          <TextField
            label="Color"
            type="color"
            value={newTagColor}
            onChange={(e) => setNewTagColor(e.target.value)}
            fullWidth
            margin="normal"
          />
        </DialogContent>
        <DialogActions>
          <Button onClick={() => setOpenDialog(false)}>Cancel</Button>
          <Button 
            onClick={handleCreateTag}
            disabled={creatingTag || !newTagName.trim()}
          >
            {creatingTag ? <CircularProgress size={24} /> : 'Create'}
          </Button>
        </DialogActions>
      </Dialog>
    </Box>
  );
});

export default TagManager;